/*
 * The UDS Collection
 * Copyright (C) 1999, 2000, 2001  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _uds_sentinel_hh
#define _uds_sentinel_hh


// includes

#ifdef _REENTRANT
#include <uds/thread.hh>
#endif
#include <map>

using namespace std;


namespace uds
{

/** logs memory allocations, checks for memory leaks end creates zombie
	objects.
	\author Julian v. Bock */
class Sentinel
{
public:
	/** allocates mem
		\return new memory address
		\arg throws bad_alloc */
	void*
	Allocate( size_t size );
	
	/** releases allocated memory
		\param p previously allocated mem
		\arg throws uds::zombie_object */
	void
	Release( void* p );
	
	/** checks whether memory at a given address was previously allocated
		\param p address
		\return true if memory was allocated */
	bool
	Allocated( const void* p ) const;
	
	/** performs the check for memory leaks */
	void
	Check();
	
private:
	typedef map< const void*, pair< const void*, int > > HeapMap;
	
	/// holds allocated memory addresses / owners
	HeapMap owner;
#ifdef _REENTRANT
	/// mutex
	Mutex	mut;
#endif
	
	/** default constructor */
	Sentinel();
	
	/** destructor */
	~Sentinel();
	
	friend Sentinel& sentinel();
};


/** \return the one and only Sentinel instance */
inline Sentinel&
sentinel()
{
	static char _s[sizeof( Sentinel )];
	static Sentinel* s = new ( _s ) Sentinel();
	return *s;
}

}

#endif // _uds_sentinel_hh
